size_t pfread(void *ptr, size_t size, size_t nmemb, FILE *stream, int compressed) {
    if(compressed) {
        unsigned char in[size * nmemb];
        static unsigned char *out;
        static int sizeOut;
        int i, toRead, recovery, isEnd;
        char *p_write, *tmpOut;
        z_stream strm;

        strm.zalloc = Z_NULL;
        strm.zfree = Z_NULL;
        strm.opaque = Z_NULL;
        strm.avail_in = 0;
        strm.next_in = Z_NULL;

        /* Taille de ce que reclame le pfread */
        toRead = size * nmemb;

        /* Il n'y a aucune donnee restante de la derniere lecture */
        if(out != NULL) {

            /* Si ce qu'on veut lire est plus grand que ce qui est
             * actuellement dans le buffer, on il faut tout
             * recuperer du buffer et demander la lecture de ce qui
             * manque. Sinon, il faut juste prendre dans le buffer
             * ce qui nous intéresse. */
            recovery = toRead > sizeOut ? sizeOut : toRead;

            /* Soit il reste le delta a lire dans l'archive, soit il
             * n'y a plus rien à lire (toRead = 0) */
            toRead -= recovery;

            /* Copie de ce qui est a recuperer en tete du pointer de
             * retour */
            memcpy(ptr, out, recovery);

            /* Le buffer doit etre reduit d'autant que ce qui a ete
             * recupere */
            sizeOut -= recovery;

            /* Si en etant reduit, le buffer n'est toujours pas
             * vide, on reconstruit le buffer en copiant ce qui
             * reste en tete d'un nouveau pointeur, qui substitura
             * au buffer actuel */
            if((tmpOut = malloc(sizeOut)) != NULL)
                memcpy(tmpOut, out + recovery, sizeOut);

            /* Switch avec le buffer mis a jour */
            free(out);
            out = tmpOut;
        } else
            recovery = 0;

        /* Si tout a ete recupere du buffer, on arrete */
        if(toRead == 0)
            return size * nmemb;

        /* Decompression gzip (16) et zlib */
        inflateInit2(&strm, 32+MAX_WBITS);

        /* Lecture dans l'archive de ce qui reste a lire (on recuperera
         * plus que ce qu'on demande, puisque qu'on recupere du
         * compresse */
        strm.avail_in = fread(in, 1, toRead, stream);
        strm.next_in = in;

        i = 0;

        /* Lecture dans le fichier compresse jusqu'a ce qu'on a ce qu'il
         * nous faut */
        do {
            /* Le buffer grandit en fonction de ce qu'on recupere,
             * puisqu'on ne peut pas prevoir quelle sera la taille
             * des donnees une fois decompressees */
            sizeOut = toRead * ++i;
            out = realloc(out, sizeOut);
            
            strm.avail_out = toRead;

            /* Lecture a la fin de ce qui est deja lu */
            strm.next_out = out + toRead * (i - 1);

            isEnd = inflate(&strm, Z_NO_FLUSH);

        } while (strm.avail_out == 0 && !isEnd);

        inflateEnd(&strm);

        /* Taille de ce qui est a recupere dans le buffer (puisqu'on a
         * forcement recupere plus que ce qu'il fallait, en decompresse */
        toRead = (i == 1) ? toRead - strm.avail_out : toRead;

        /* Copie a la suite de ce qui eventuellement deja ete recupere
         * du buffer, de ce qu'on vient de recuperer */
        memcpy(ptr + recovery, out, toRead);

        /* Taille du buffer actuel, sans ce qui n'a pas ete utilise */
        sizeOut -= strm.avail_out - toRead;

        /* Si il y a un reliquat, on recree le buffer avec uniquement ce
         * reliquat, a la bonne taille */
        if((tmpOut = malloc(sizeOut)) != NULL)
            memcpy(tmpOut, out + toRead, sizeOut);

        /* Switch des buffers */
        free(out);
        out = tmpOut;

        isTherePfreadStock = sizeOut > 0;

        return toRead;
    } else
        return fread(ptr, size, nmemb, stream);
}
