/*
 * ptar - PROJET ESIAL RS 2011/2012
 * Copyright (C) 2011 Julien VAUBOURG / Valentin KELLER
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ptar.h"
#include <utime.h>

/*
 * Recuperation des attributs du fichier a extraire
 * @header In Entetes ustar du fichier a extraire
 * @attr Out Attributs du fichier
 * @return Nombre d'erreurs rencontrees
*/
void get_attributes(header_posix_ustar* header, options* userOptions, file_attributes* attr) {

	/* DOC : "Modification time of file, as an octal number in ASCII." */
	sscanf(header->size, "%o", &attr->size);

	/* DOC : "If the pathname is too long to fit in the 100 bytes provided
	 * by the standard format, it can be split at any / character with the
	 * first portion going into the prefix field" */
	if(header->prefix[0] != '\0')
		snprintf(
			attr->filename,
			MAX_FILENAME,
			"%s/%s",
			header->prefix, header->name
		);
	else
		strncpy(
			attr->filename,
			header->name,
			MAX_FILENAME
		);

	/* Copie de l'eventuel nom du lien */
	strncpy(
		attr->linkname,
		header->linkname,
		MAX_FILENAME
	);

	/* Copie du chemin complet du fichier a extraire */
	snprintf(attr->fn_with_path, MAX_PATH + 1 + MAX_FILENAME, "%s/%s",
		userOptions->path,
		attr->filename);
	
	sscanf(header->mode, "%o", &attr->mode);
	sscanf(header->uid, "%o", &attr->uid);
	sscanf(header->gid, "%o", &attr->gid);
	sscanf(header->mtime, "%o", &attr->mtime);
	sscanf(header->devminor, "%o", &attr->minor);
	sscanf(header->devmajor, "%o", &attr->major);
}

/* Change les attributs du fichier cree
 * @attr Attributs du fichier, avec son nom
 */
unsigned int set_attributes(file_attributes* attr) {
	struct utimbuf sutime = {
		time(NULL),
		attr->mtime
	};

	utime(attr->filename, &sutime);
	chmod(attr->filename, attr->mode);

	/* Inutile de lancer une erreur si c'est pour ne rien changer */
	if(attr->uid != getuid() || attr->gid != getgid())
		lchown(attr->filename, attr->uid, attr->gid);

	/* Erreur si on tente de changer le uid/gid sans avoir les droits root */
	if(errno == EPERM) {
		fprintf(
			stderr,
			"[ERREUR] %s : vous devez avoir des droits root pour changer le uid ou/et le gid\n",
			attr->fn_with_path
		);

		return 1;
	}

	return 0;
}

/*
 * Verifie si le checksum du header est valide
 * @header Entetes du fichier avec le checksum
 * @return Checksum valide ou non
 */
unsigned int isValid_checksum(header_posix_ustar header) {
	unsigned int i, sum1, sum2;
	char *p, *p_end;

	/* DOC: "Header checksum, stored as an octal number in ASCII." */
	sscanf(header.checksum, "%o", &sum1);

	/* DOC: "To compute the checksum, set the checksum field to all spaces" */
	for(
		i = 0;
		i < sizeof(header.checksum);
		header.checksum[i++] = ' '
	);

	p = (char *) &header;
	p_end = (char *) (p + sizeof(header));

	/* DOC: "then sum all bytes in the header using unsigned arithmetic" */
	for(
		sum2 = 0;
		p < p_end;
		sum2 += *p++
	);

	return sum1 == sum2;
}
