/*
 * ptar - PROJET ESIAL RS 2011/2012
 * Copyright (C) 2011 Julien VAUBOURG / Valentin KELLER
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ptar.h"

/*
 * Extrait un lien symbolique ou physique
 * @header_t Informations en provenance du thread
 * @type Type de lien (LINK_SYM ou LINK_HARD)
 * @return Nombre d'erreurs rencontrees
 */
unsigned int extract_link(header_for_thread* header_t, unsigned int type) {
	char link_with_path[MAX_PATH + 1 + MAX_FILENAME];
	char *p_filename, *p_linkname;

	/* Nom du lien avec le chemin complet */
	snprintf(link_with_path, MAX_PATH + 1 + MAX_FILENAME, "%s/%s",
		header_t->userOptions->path,
		header_t->attr.linkname);

	/* Creation du lien */
	if(type == LINK_SYM) {
		p_filename = header_t->attr.fn_with_path;
		p_linkname = link_with_path;

		/* Un lien symbolique impose un chemin relative a son
		 * emplacement, ou bien absolu au systeme. On supprime donc les
		 * parties communes pour creer un chemin relatif */
		while(*p_filename++ == *p_linkname++);

		symlink(--p_linkname, header_t->attr.fn_with_path);

	} else {

		/* Attend que le fichier du lien soit cree par un autre thread,
		 * c'est la seule solution possible */
		while(access(link_with_path, F_OK) != 0)
			usleep(10);

		link(link_with_path, header_t->attr.fn_with_path);
	}

	return 0;
}
