/*
 * ptar - PROJET ESIAL RS 2011/2012
 * Copyright (C) 2011 Julien VAUBOURG / Valentin KELLER
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ptar.h"

/*
 * Extrait un fichier regulier de l'archive en cours
 * @header_t Informations en provenance du thread
 * @return Nombre d'erreurs rencontrees
 */
unsigned int extract_regular(header_for_thread* header_t) {
	FILE* extractedFile;
	unsigned int nbBlocks, nbBuffers, i;
	char buf[RECORD_SIZE * BUFFER_SIZE];

	/* Creation du fichier a extraire */
	extractedFile = fopen(header_t->attr.fn_with_path, "w");

	/* Impossible d'ecrire le fichier */
	if(extractedFile == NULL)
		return 1;

	/* Calcul du nombre total d'enregistrements a lire pour obtenir
	 * l'integralite des donnees du fichier, en arrondissant au bloc
	 * superieur */
	nbBlocks = header_t->attr.size / RECORD_SIZE + (header_t->attr.size % RECORD_SIZE != 0);

	/* Le fichier n'est pas vide */
	if(nbBlocks > 0) {

		/* Nombre de tampons qu'il faudra remplir et vider pour creer le
		 * fichier, arrondi au tampon superieur */
		nbBuffers = nbBlocks / BUFFER_SIZE + (nbBlocks % BUFFER_SIZE != 0);

		/* Completion du fichier, avec le tampon. Si c'est un
		 * petit fichier (plus petit que RECORD_SIZE*BUFFER_SIZE)
		 * la boucle n'aura aucune iteration et le seul write
		 * qu'il y aura sera donc dans le thread de synchro */
		for(i = 0; i < nbBuffers - 1; i++) {
			pfread_sem(buf, RECORD_SIZE * BUFFER_SIZE, 1, header_t->tarFile, header_t->sem, &(header_t->cursor));
			fwrite(buf, RECORD_SIZE * BUFFER_SIZE, 1, extractedFile);
		}

		/* Calcul du nombre d'enregistrements à lire pour faire avancer
		 * le curseur du fichier jusqu'au debut du prochain
		 * enregistrement */
		int remainingBlocksToRead = nbBlocks - (nbBuffers - 1) * BUFFER_SIZE;

		/* Calcul du nombre d'octets a ecrire, pour completer le
		 * fichier, sans ecrire les caracteres NULs dus a la fin de
		 * l'enregistrement ou du bloc. DOC : "For compatibility with
		 * tape drives that use fixed block sizes, programs that read or
		 * write tar files always read or write a fixed number of
		 * records with each I/O operation." */
		int remainingBytesToWrite = header_t->attr.size - ((nbBuffers - 1) * BUFFER_SIZE * RECORD_SIZE);

		/* Ecriture de la fin du fichier (ou du fichier dans son
		 * integralite s'il est petit) depuis le thread qui sera
		 * lance ensuite pour la synchro */
		pfread_sem(buf, RECORD_SIZE, remainingBlocksToRead, header_t->tarFile, header_t->sem, &(header_t->cursor));
		fwrite(buf, 1, remainingBytesToWrite, extractedFile);
	}

	return 0;
}
