/*
 * ptar - PROJET ESIAL RS 2011/2012
 * Copyright (C) 2011 Julien VAUBOURG / Valentin KELLER
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ptar.h"

/*
 * Recupere les options de la ligne de commande
 * @userOptions Enregistrement des options
 * @agc Argc du main
 * @agv Argv du main
 */
void get_options(options* userOptions, int argc, char** argv){
	int optch;
	extern int opterr;
	DIR* path_dir;
	char format[] = "vtp:C:";

	opterr = 1;

	userOptions->verbose = 0;
	userOptions->list = 0;
	userOptions->nbThreads = NB_DEFAULT_THREADS;
	strcpy(userOptions->path, ".");
		
	while((optch = getopt(argc, argv, format)) != -1) {
		switch(optch) {  
			case 'v':  
				userOptions->verbose = 1;
			break;

			case 't':  
				userOptions->list = 1;  
			break;

			case 'p':
				userOptions->nbThreads = atoi(optarg);
			break;

			case 'C':
				strncpy(userOptions->path, optarg, MAX_PATH);
			break;

			default:  
				printf("\n%s -[vhpCl] fichier_tar|fichier_tar_gz\n", argv[0]);
				puts(" Options :");
				puts(" -v : Mode verbeux");
				printf(" -p : Nombre de threads (defaut : %d)\n", NB_DEFAULT_THREADS);
				puts(" -C : Chemin d'extraction (defaut : ./)");
				puts(" -l : Lister les noms des fichiers");
				puts(" -h : Afficher cette aide\n");

				exit(EXIT_SUCCESS);
			break;
		}
	}

	/* Pas de nom de fichier indique */
	if(argv[optind] == NULL) {
		fprintf(stderr, "[FATAL] Vous devez indiquer un nom de fichier (%s -h pour plus d'aide)\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	strncpy(userOptions->source, argv[optind], MAX_FILENAME);

	/* Verification du chemin d'extraction */
	if((path_dir = opendir(userOptions->path)) == NULL) {
		closedir(path_dir);
		fprintf(stderr, "[FATAL] Un probleme a ete rencontre lors de l'ouverture du chemin d'extraction indique\n");
		exit(EXIT_FAILURE);
	}

	closedir(path_dir);

	/* Verification du nombre indique pour les threads */
	if(userOptions->nbThreads < 1) {
		fprintf(stderr, "[FATAL] Le nombre de threads doit etre superieur a zero\n");
		exit(EXIT_FAILURE);
	}
}

/* 
 * Fonction d'arret de l'horloge permettant de calculer le delai d'extraction
 * @tic Valeur de debut du chrono
 */
void stopTimer(struct timeval* tic){
	struct timeval tac;
	gettimeofday(&tac, NULL);

	long delai = (tac.tv_sec * 1000000 + tac.tv_usec) - (tic->tv_sec * 1000000 + tic->tv_usec);

	int h = delai / 3600000000;
	int min = (delai / 60000000) % 3600000000;
	int s = delai / 1000000;
	int m = (delai / 1000) % 1000;
	int u = delai % 1000;

	printf("(delai : %dh %dmin %ds %dms et %dus)\n", h, min, s, m, u);
}
