/*
 * ptar - PROJET ESIAL RS 2011/2012
 * Copyright (C) 2011 Julien VAUBOURG / Valentin KELLER
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>
#include <dlfcn.h>
#include <semaphore.h>
#include <dirent.h>
#include <zlib.h>

/* DOC : "A tar archive consists of a series of 512-byte records" */
#define RECORD_SIZE 512

/* Nombre d'enregistrements dans le tampon, selon la RAM */
#define BUFFER_SIZE 8

/* Nombre de threads par defaut */
#define NB_DEFAULT_THREADS 8

/* Taille max d'un nom de fichier */
#define MAX_FILENAME 256

/* Taille max pour un chemin */
#define MAX_PATH 10*MAX_FILENAME

/* Etats pour threadsAlive */
#define FREE_THREAD 0
#define RUNNING_THREAD 1
#define DEAD_THREAD -1

/* Types de fichier */
#define FILE_REGULAR 0
#define FILE_HARD 1
#define FILE_SYM 2
#define FILE_CDEVICE 3
#define FILE_BDEVICE 4
#define FILE_DIR 5
#define FILE_FIFO 6

/* Types de liens */
#define LINK_SYM 0
#define LINK_HARD 1

/* POSIX ustar Archives */
typedef struct {
	char name[100];
	char mode[8];
	char uid[8];
	char gid[8];
	char size[12];
	char mtime[12];
	char checksum[8];
	char typeflag[1];
	char linkname[100];
	char magic[6];
	char version[2];
	char uname[32];
	char gname[32];
	char devmajor[8];
	char devminor[8];
	char prefix[155];
	char pad[12];
} header_posix_ustar;

typedef struct {
	char verbose, list;
	char source[MAX_FILENAME];
	char path[MAX_PATH];
	unsigned int nbThreads;
} options;

typedef struct {
	char filename[MAX_FILENAME], linkname[MAX_FILENAME];
	char fn_with_path[MAX_PATH + 1 + MAX_FILENAME];
	int size, mode, uid, gid, mtime, minor, major;
} file_attributes;

typedef struct {
	file_attributes attr;
	FILE* tarFile;
	unsigned int iThread;
	int typeflag;
	options* userOptions;
	long cursor;
	sem_t* sem;
} header_for_thread;

typedef struct {
	unsigned int isGzipped;
	void* dynopen;
	gzFile (*gzopen) (const char *path, const char *mode);
	int (*gzread) (gzFile file, voidp buf, unsigned len); 
	int (*gzclose) (gzFile file);
	int (*gzeof) (gzFile file);
	int (*gzseek) (gzFile file, int offset, int whence); 
	int (*gztell) (gzFile file);
} zlib;

unsigned int* threadsAlive;
zlib compressed;

void get_attributes(header_posix_ustar* header, options* userOptions, file_attributes* attr);
unsigned int set_attributes(file_attributes* attr);
unsigned int isValid_checksum(header_posix_ustar header);
int join_thread(pthread_t thread);
int isGzip(FILE* tarFile);
void loaddyn_zlib();
FILE* pfopen(char* filename);
size_t pfread(void *ptr, size_t size, size_t nmemb, FILE *stream);
int pfclose(FILE* tarFile);
int pfeof(FILE* tarFile);
int pfseek(FILE* tarFile, int offset, int whence);
int pftell(FILE* tarFile);
size_t pfread_sem(void* ptr, size_t size, size_t nmemb, FILE *stream, sem_t* semaphore, long* cursor);

void extract_files(void* p_data);
unsigned int extract_regular(header_for_thread* header_t);
unsigned int extract_link(header_for_thread* header_t, unsigned int type);
unsigned int extract_device(header_for_thread* header_t, unsigned int type);
unsigned int extract_dir(header_for_thread* header_t);
unsigned int extract_dirs(char* filename);
unsigned int extract_fifo(header_for_thread* header_t);

void get_options(options* myOption, int argc, char** argv);
void stopTimer(struct timeval* tic);
